package com.palserv.component;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;

/**
 * When this class is properly instantiated, it displays
 * a calendar on the screen.  The current date is displayed
 * differently (lowered button) from the other dates and
 * dates outside the current month are grayed out and
 * not selectable.
 * <p>
 * The user may click on any day of the displayed month
 * to select that date.  "Arrow" buttons allow the user
 * to go to the next month, previous month, next year, and
 * previous year.
 * <p>
 * Although it was designed to be used with the DateBox
 * class, it could be used separately.	A class which
 * wants to use this class should implement the CallPalCal
 * interface.  I would recommend using DateBox as an
 * example if you plan to call this class independently.
 */
public class PalCal extends JWindow {
	String useDate;
//	  int selectedDay;
	CallPalCal creator;
	JFrame aFrame;
	DateFormatter df;

/**
 * Constructs a PalCal object which can be controlled
 * (owned) by an object of a class which implements
 * the CallPalCal interface.
 */
	public PalCal(CallPalCal o) {
		creator = o;
		df = o.getDF();
		useDate = o.getUseDate();
//		  setContentPane(makeContentPane());
	}
/*
	public PalCal() {
		creator = null;
	}
*/
/*
	public static void main(String[] args) {

		PalCal pc = new PalCal();
		pc.aFrame = new JFrame("PalCal");
		pc.aFrame.addWindowListener(
			new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					System.exit(0);
				}
			}
		);
		pc.getArgs(args);
		Container apane = pc.makeContentPane();
		pc.aFrame.setContentPane(apane);

//		aFrame.setSize(149,233);
		pc.aFrame.pack();
		pc.aFrame.setVisible(true);
	}

	void getArgs(String[] args) {
		if (args.length < 1) {
			useDate = df.getDateString(6,10,1946);
		} else {
			useDate = args[0];
		}
	}
*/

/**
 * Build the main GUI.
 */
	JPanel makeContentPane() {
		try {
			UIManager.setLookAndFeel(
				UIManager.getSystemLookAndFeelClassName());
		} catch(Exception e) {
			rpt("Unable to set system laf: "+e.getMessage());
		}
		int[] iDte;

		iDte = df.parseDate(useDate);
		CalVO cv = getCalData(iDte[0], iDte[1], iDte[2]);
// spt("contents of cv: "+cv);
		JPanel main = new JPanel();
		main.setLayout(new BorderLayout());
		main.setBorder(new BevelBorder(BevelBorder.RAISED));
//		  JPanel top = new JPanel();
		Box top = Box.createHorizontalBox();
		JPanel mid = new JPanel();

		main.add("North", top);
		main.add("Center", mid);
		JButton jbLastYear = new JButton("<<");
		JButton jbLastMonth = new JButton("<");
		JButton jbNextMonth = new JButton(">");
		JButton jbNextYear = new JButton(">>");
		addButList(jbLastYear, cv, -12);
		addButList(jbNextYear, cv, 12);
		addButList(jbLastMonth, cv, -1);
		addButList(jbNextMonth, cv, 1);
		top.add(Box.createHorizontalGlue());
		top.add(jbLastYear);
		top.add(jbLastMonth);
		top.add(Box.createHorizontalGlue());
		JLabel monYear = new JLabel(cv.monStr+" "+cv.year);
		monYear.setFont(new Font("SansSerif", Font.BOLD, 12));
		top.add(monYear);
		top.add(Box.createHorizontalGlue());
		top.add(jbNextMonth);
		top.add(jbNextYear);
		top.add(Box.createHorizontalGlue());

		mid.setLayout(new GridLayout(cv.numWeeks + 1, 7));
// int cvn = cv.numWeeks + 1;
// spt("Should be "+ cvn + " rows");
		String[] days = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
		for (int i = 0; i < 7; i ++) {
			JLabel jl = new JLabel(days[i]);
			jl.setFont(new Font("SansSerif", Font.BOLD, jl.getFont().getSize()));
			jl.setHorizontalAlignment(SwingConstants.CENTER);
			mid.add(jl);
		}

		int tndfw = 7 - cv.numDaysShowingPriorMonth;
		int tndmfw = cv.daysInMonth - tndfw;
		int tnwwpfw = tndmfw / 7;
		int sdpm = cv.startDayPriorMonth;
		int day = 1;
		if	(cv.numDaysShowingPriorMonth > 0) {
			for (int i = 0; i < cv.numDaysShowingPriorMonth; i ++) {
				makBut(mid, sdpm, false, cv);
				sdpm ++;
			}
			for (int i = 0; i < (7 - cv.numDaysShowingPriorMonth); i ++) {
				makBut(mid, day, true, cv);
				day ++;
			}
		} else {
			tnwwpfw ++;
		}
		for (int i = 0; i < tnwwpfw; i ++) {
			for (int j = 0; j < 7; j ++) {
				makBut(mid, day, true, cv);
				day ++;
			}
		}
		if (cv.numDaysNextMonth > 0) {
			for (int i = 0; i < cv.numDaysNextMonth; i ++) {
				makBut(mid, day, true, cv);
				day ++;
			}
			for (int i = 0; i < (7 - cv.numDaysNextMonth); i ++) {
				makBut(mid, i + 1, false, cv);
			}
		}

		return main;
	}

/**
 * Add a button for changing the date ("<<", "<", ">", and ">>")
 */
	void addButList(JButton jb, final CalVO cv, final int x) {
		jb.setFont(new Font("SansSerif", Font.BOLD, jb.getFont().getSize()));
		jb.setBorder(new BevelBorder(BevelBorder.RAISED));
		jb.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				changeDate(cv, x);
			}
		});
	}

/**
 * Called by addButList method when button hit to change the
 * date.
 * @param cv a CalVO object containing date information
 * @param x the number of months to add (could be negative)
 */
	void changeDate(CalVO cv, int x) {
		int month = cv.month;
		int day = cv.day;
		int year = cv.year;
		month += x + 1;
		if (month < 0) {
			month += 12;
			year --;
		}
		if (month > 11) {
			month -= 12;
			year ++;
		}
// spt("PalCal.changeDate: "+x+"; oldDate = "+cv.month+"/"+cv.year+"; new date = "+month+"/"+year);
//		  useDate = ""+month+"/"+day+"/"+year;
		useDate = df.getDateString(month, day, year);
		if (creator == null) {
			this.aFrame.setContentPane(makeContentPane());
			this.aFrame.pack();
			this.aFrame.setVisible(true);
		} else {
			this.setContentPane(makeContentPane());
			this.pack();
			this.setVisible(true);
		}
	}

/**
 * This method makes one "day" button for the calendar face.
 * @param jp the JPanel in which the button will reside
 * @param dnum the number for the face of the button
 * @param inMonth whether this number is in the current month
 * @param cv a CalVO object containing date information
 */
	void makBut(JPanel jp, final int dnum, boolean inMonth, final CalVO cv) {
		JButton jb = new JButton(""+dnum);
		jp.add(jb);
		if (inMonth) {
//			  jb.setFont(new Font("SansSerif", Font.BOLD, 12));
//			  Font f = jb.getFont();
 // 		  int siz = f.getSize();
			jb.setFont(new Font(null, Font.BOLD, jb.getFont().getSize()));
			if (dnum == cv.day) {
				jb.setBorder(new BevelBorder(BevelBorder.LOWERED));
			} else {
				jb.setBorder(null);
			}
			jb.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent ae) {
					setSelected(cv.month, dnum, cv.year);
				}
			});
		} else {
			jb.setForeground(Color.gray);
			jb.setBorder(null);
		}
	}

/**
 * This method is called when the user clicks a "day" button.
 * We pass the result back to the CallPalCal class from which
 * this class was instantiated.
 * @param m month of the selection
 * @param d day of the selection
 * @param y year of the selection
 */
	void setSelected(int m, int d, int y) {
		m ++;
		if (creator == null) {
			spt("day "+d+" selected");
		} else {
//			  creator.setResponse(""+m+"/"+d+"/"+y);
			creator.setResponse(df.getDateString(m, d, y));
		}
	}

/**
 * utility method
 */
	static void rpt(String s) {
		System.out.println(s);
		System.exit(0);
	}

/**
 * This method uses the Calendar and GregorianCalendar classes
 * of java.util to get information about the date passed
 * as arguments.
 * <p>
 * It produces a CalVO object which has all the data needed
 * to build and display a calendar.
 * @param m month
 * @param d day
 * @param yr year
 * @return CalVO containing calendar data
 */
	CalVO getCalData(int m, int d, int yr) {
		CalVO cv = new CalVO();
		cv.month = m;
		cv.day = d;
		cv.year = yr;
		Calendar calendar = new GregorianCalendar(yr, m, d);
		int dow = calendar.get(Calendar.DAY_OF_WEEK);
		int dom = calendar.get(Calendar.DAY_OF_MONTH);
//		  int off = (7 - ((dom + 7 - dow) % 7)) % 7;
		cv.numDaysShowingPriorMonth = (7 - ((dom + 7 - dow) % 7)) % 7;
		// day of week of first of month
		int dowofom = cv.numDaysShowingPriorMonth + 1;
		// number of days in month
//		  int ndim = getDim(m, yr);
		cv.daysInMonth = getDim(m, yr);
// spt("there are "+ndim+ " days in the month");
		// number of days in first week
		int tndfw = 7 - cv.numDaysShowingPriorMonth;
// spt("there are "+tndfw+ " days in the first week");
		// number of days minus first week
		int tndmfw = cv.daysInMonth - tndfw;
// spt("there are "+tndmfw+ " days left in month (after the first week)");
		// number of whole weeks past first (incomplete) week
		int tnwwpfw = tndmfw / 7;
// spt("there are "+tnwwpfw+ " whole weeks past first week");
		// number of days left over
//		  int tndlo = (tndmfw - (tnwwpfw * 7)) % 7;
		cv.numDaysNextMonth = (tndmfw - (tnwwpfw * 7)) % 7;
// spt("there are "+tndlo+ " days left over");
		// number of weeks to show
		cv.numWeeks = tnwwpfw + (cv.numDaysNextMonth == 0?0:1) + (tndfw == 0?0:1);
//		  int nws = tnwwpfw + (tndlo == 0?0:1) + (tndfw == 0?0:1);
// spt("there are "+nws+ " weeks to show");
//		  spt("offset="+off+"; mon="+m+"; day="+d+"; dow="+dow+"; dom="+dom+"; dowofom="+dowofom);
		cv.monStr = getMonName(m);
//		  spt(getMonName(m)+" "+yr);
//		  spt("S  M  T  W  T  F  S");
//		  int day = 1;
//		  StringBuffer sb = new StringBuffer();
		if	(cv.numDaysShowingPriorMonth > 0) {
			int mm = m;
			int yrr = yr;
			mm --;
			if (mm < 0) {
				mm = 12;
				yrr --;
			}
//			  int ndlm = getDim(mm, yrr);
			cv.numDaysPriorMonth = getDim(mm, yrr);
			cv.startDayPriorMonth =
				cv.numDaysPriorMonth - cv.numDaysShowingPriorMonth + 1;
/*
			for (int i = 0; i < cv.numDaysShowingPriorMonth; i ++) {
				sb.append(numForm(cv.startDayPriorMonth,2)+" ");
				cv.startDayPriorMonth ++;
			}
			for (int i = 0; i < (7 - cv.numDaysShowingPriorMonth); i ++) {
				sb.append(numForm(day,2)+" ");
				day ++;
			}
		} else {
			tnwwpfw ++;
*/
		}
/*
		spt(sb.toString());
		sb.delete(0,sb.length());
		for (int i = 0; i < tnwwpfw; i ++) {
			for (int j = 0; j < 7; j ++) {
				sb.append(numForm(day,2) + " ");
				day ++;
			}
			spt(sb.toString());
			sb.delete(0,sb.length());
		}
		if (tndlo > 0) {
			for (int i = 0; i < tndlo; i ++) {
				sb.append(numForm(day,2) + " ");
				day ++;
			}
			for (int i = 0; i < (7 - tndlo); i ++) {
				sb.append(numForm(i + 1,2) + " ");
			}
			spt(sb.toString());
			sb.delete(0,sb.length());
		}
*/
		return cv;
	}

/**
 * return a string of "n" of length "l"
 * right justified, padded to the left with zeros
 */
	public static String numForm(int n, int l) {
		Integer i = new Integer(n);
		String s = i.toString();
		String p = "0000000000000000000000" + s;
		int nl = p.length();
		int x = nl - l;
// spt("numForm: p = "+p+"; l = "+l+"; nl = "+nl+"; x = "+x);
		String r = p.substring(x, nl);
		return r;
	}

/**
 * utility routine
 */
	void spt(String s) {
		System.out.println(s);
	}

/**
 * "Look up" the 3-letter month abbreviation.
 * @param m month (0 = January, 11 = December)
 * @return the name.
 */
	String getMonName(int m) {
		String[] mon = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
			"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
		return mon[m];
	}

/**
 * figure out the number of days in a particular month
 * @param mon month
 * @param yr year
 * @return number of days in the specified month
 */
	int getDim(int mon, int yr) {
		int[] n = new int[] {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
		int dim =  n[mon];
		if (mon == 1) { // Feb
			if (isLeap(yr)) {
				dim ++;
			}
		}
		return dim;
	}

/**
 * figure out whether or not a year is a leap year
 * @param yr year
 * @return true if year is a leap year
 */
	boolean isLeap(int yr) {
		boolean ret = false;
		if ((yr % 4) == 0) {	// divisible by 4
			if ((yr % 100) == 0) {	// divisible by 100
				if ((yr % 1000) == 0) {  // divisible by 1000
					// div by 4, and by 100, and by 1000
					return true;
/*
				} else {
					// div by 4, and by 100, but not by 1000
					return false;
*/
				}
			} else {
				// div by 4, not by 100
				return true;
			}
/*
		} else {
			// not div by 4
			return false;
*/
		}
		return ret;
	}
}

/**
 * This class is used internally as a value object to
 * encapsulate information about a particular date.
 */
class CalVO {
	int year;
	int month;
	int day;
	String monStr;
	int numDaysPriorMonth;
	int numDaysShowingPriorMonth;
	int startDayPriorMonth;
	int numWeeks;
	int numDaysNextMonth;
	int daysInMonth;

/**
 * utility method (for debugging only)
 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("Year="+year);
		sb.append("; Month="+month);
		sb.append("; Day="+day);
		sb.append("; mon="+monStr);
		sb.append("; numDaysPriorMonth="+numDaysPriorMonth);
		sb.append("; numDaysShowingPriorMonth="+numDaysShowingPriorMonth);
		sb.append("; startDayPriorMonth="+startDayPriorMonth);
		sb.append("; numWeeks="+numWeeks);
		sb.append("; numDaysNextMonth="+numDaysNextMonth);
		sb.append("; daysInMonth="+daysInMonth);
		return sb.toString();
	}
}
